#pylint: disable=C0301, C0103, W0212, W0401

"""
.. module:: pilot
   :platform: Unix
   :synopsis: RADICAL-Pilot is a distributed Pilot-Job framework.

.. moduleauthor:: Ole Weidner <ole.weidner@rutgers.edu>
"""

__copyright__ = "Copyright 2013-2014, http://radical.rutgers.edu"
__license__   = "MIT"


# ------------------------------------------------------------------------------
# constants and types
from .types      import *
from .states     import *
from .logentry   import * 
from .scheduler  import *
from .constants  import *
from .exceptions import *

# ------------------------------------------------------------------------------
#
from .session    import Session 
from .context    import Context

from .unit_manager              import UnitManager
from .compute_unit              import ComputeUnit
from .compute_unit_description  import ComputeUnitDescription

from .pilot_manager             import PilotManager
from .compute_pilot             import ComputePilot
from .compute_pilot_description import ComputePilotDescription

from .resource_config    import ResourceConfig
from .staging_directives import COPY, LINK, MOVE, TRANSFER, SKIP_FAILED, CREATE_PARENTS

from .utils import version, version_detail, version_branch
from .utils import sdist_name, sdist_path
from .utils import logger

# ------------------------------------------------------------------------------





# ------------------------------------------------------------------------------
#
# global constants
#
UMGR_SCHEDULING_QUEUE          = 'umgr_scheduling_queue'
UMGR_STAGING_INPUT_QUEUE       = 'umgr_staging_input_queue'
UMGR_STAGING_OUTPUT_QUEUE      = 'umgr_staging_output_queue'

UMGR_SCHEDULING_COMPONENT      = 'UmgrSchedulingComponent'
UMGR_STAGING_INPUT_COMPONENT   = 'UmgrStagingInputComponent'
UMGR_STAGING_OUTPUT_COMPONENT  = 'UmgrStagingOutputComponent'
UMGR_UPDATE_WORKER             = 'UmgrUpdateWorker'

AGENT_STAGING_INPUT_QUEUE      = 'agent_staging_input_queue'
AGENT_SCHEDULING_QUEUE         = 'agent_scheduling_queue'
AGENT_EXECUTING_QUEUE          = 'agent_executing_queue'
AGENT_STAGING_OUTPUT_QUEUE     = 'agent_staging_output_queue'

AGENT_STAGING_INPUT_COMPONENT  = 'AgentStagingInputComponent'
AGENT_SCHEDULING_COMPONENT     = 'AgentSchedulingComponent'
AGENT_EXECUTING_COMPONENT      = 'AgentExecutingComponent'
AGENT_STAGING_OUTPUT_COMPONENT = 'AgentStagingOutputComponent'
AGENT_UPDATE_WORKER            = 'AgentUpdateWorker'
AGENT_HEARTBEAT_WORKER         = 'AgentHeartbeatWorker'

PMGR_LAUNCHING_QUEUE           = 'pmgr_launching_queue'

PMGR_LAUNCHING_COMPONENT       = 'PmgrLaunchingComponent'

UMGR_STATE_PUBSUB              = 'umgr_state_pubsub'

AGENT_UNSCHEDULE_PUBSUB        = 'agent_unschedule_pubsub'
AGENT_RESCHEDULE_PUBSUB        = 'agent_reschedule_pubsub'
AGENT_COMMAND_PUBSUB           = 'agent_command_pubsub'
AGENT_STATE_PUBSUB             = 'agent_state_pubsub'




#!/usr/bin/env python

# This is an example MPI4Py program that is used
# by different examples and tests.

import sys
import time
import traceback

from mpi4py import MPI

try :
    print "start"
    SLEEP = 10
    name  = MPI.Get_processor_name()
    comm  = MPI.COMM_WORLD

    print "mpi rank %d/%d/%s"  % (comm.rank+1, comm.size, name)

    time.sleep(SLEEP)

    comm.Barrier()   # wait for everybody to synchronize here

except Exception as e :
    traceback.print_exc ()
    print "error : %s" % e
    sys.exit (1)

finally :
    print "done"
    sys.exit (0)




#!/usr/bin/env python

__copyright__ = "Copyright 2013-2014, http://radical.rutgers.edu"
__license__   = "MIT"

import os
import sys

import radical.pilot as rp
import radical.utils as ru

# READ: The RADICAL-Pilot documentation: 
#   http://radicalpilot.readthedocs.org/
#
# Try running this example with RADICAL_PILOT_VERBOSE=debug 
# set if you want to see what happens behind the scences!


RUNTIME  =    20  # how long to run the pilot
CORES    =    64  # how many cores to use for one pilot
UNITS    =   128  # how many units to create
SLEEP    =     0  # how long each unit sleeps
SCHED    = rp.SCHED_DIRECT_SUBMISSION


#------------------------------------------------------------------------------
#
if __name__ == "__main__":

    # we use a reporter class for nicer output
    report = ru.Reporter("Getting Started")

    # Create a new session. No need to try/except this: if session creation
    # fails, there is not much we can do anyways...
    session = rp.Session()

    # all other pilot code is now tried/excepted.  If an exception is caught, we
    # can rely on the session object to exist and be valid, and we can thus tear
    # the whole RP stack down via a 'session.close()' call in the 'finally'
    # clause...
    try:
        report.info('read configs')
        resources = ru.read_json('%s/config.json', os.path.dirname(__file__))
        report.ok('\\ok\n')

        report.header('submit pilots')

        # prepare some input files for the compute units
        os.system ('hostname > file1.dat')
        os.system ('date     > file2.dat')

        # Add a Pilot Manager. Pilot managers manage one or more ComputePilots.
        pmgr = rp.PilotManager(session=session)

        # Define an [n]-core local pilot that runs for [x] minutes
        pdescs = list()
        for resource in sys.argv[1:]:
            pdesc = rp.ComputePilotDescription()
            pdesc.resource      = resource
            pdesc.cores         = CORES
            pdesc.project       = resources[resource]['project']
            pdesc.queue         = resources[resource]['queue']
            pdesc.runtime       = RUNTIME
            pdesc.cleanup       = False
            pdesc.access_schema = resources[resource]['schema']
            pdescs.append(pdesc)

        # Launch the pilot.
        pilots = pmgr.submit_pilots(pdescs)
    
        report.header('submit units')

        session.close (cleanup=False)

    # fetch profiles and convert into inspectable data frames
    if session:
        session_id = session.uid


    : 

    import radical.pilot.utils as rpu
    # we have a session
    profiles   = rpu.fetch_profiles(sid=session_id, tgt='/tmp/')
    profile    = rpu.combine_profiles (profiles)
    frame      = rpu.prof2frame(profile)
    sf, pf, uf = rpu.split_frame(frame)

  # print len(sf)
  # print len(pf)
  # print len(uf)
  # 
  # print sf[0:10]
  # print pf[0:10]
  # print uf[0:10]
    rpu.add_info(uf)
    rpu.add_states(uf)
    adv = uf[uf['event'].isin(['advance'])]
    print len(adv)
  # print uf[uf['uid'] == 'unit.000001']
  # print list(pf['event'])

    rpu.add_frequency(adv, 'f_exe', 0.5, {'state' : 'Executing', 'event' : 'advance'})
    print adv[['time', 'f_exe']].dropna(subset=['f_exe'])

    s_frame, p_frame, u_frame = rpu.get_session_frames(session_id)
    print str(u_frame)

    for f in profiles:
        os.unlink(f)


#-------------------------------------------------------------------------------




#!/usr/bin/env python

__copyright__ = "Copyright 2013-2014, http://radical.rutgers.edu"
__license__   = "MIT"

import sys
import radical.pilot as rp


""" DESCRIPTION: A Simple Workload consisting one task of parallel k-means clustering.
"""

# READ: The RADICAL-Pilot documentation: 
#   http://radicalpilot.readthedocs.org/en/latest
#
# Try running this example with RADICAL_PILOT_VERBOSE=debug set if 
# you want to see what happens behind the scences!


#------------------------------------------------------------------------------
#
def pilot_state_cb (pilot, state):

    if not pilot:
        return

    print "[Callback]: ComputePilot '%s' state: %s." % (pilot.uid, state)

    if state == rp.FAILED:
        sys.exit (1)


#------------------------------------------------------------------------------
#
def unit_state_cb (unit, state):

    if not unit:
        return

    global CNT

    print "[Callback]: unit %s on %s: %s." % (unit.uid, unit.pilot_id, state)

    if state == rp.FAILED:
        print "stderr: %s" % unit.stderr
        sys.exit(2)


#------------------------------------------------------------------------------
#
if __name__ == "__main__":

    # we can optionally pass session name to RP
    if len(sys.argv) > 1:
        session_name = sys.argv[1]
    else:
        session_name = None

    # Create a new session. No need to try/except this: if session creation
    # fails, there is not much we can do anyways...
    session = rp.Session(name=session_name)
    print "session id: %s" % session.uid

    # all other pilot code is now tried/excepted.  If an exception is caught, we
    # can rely on the session object to exist and be valid, and we can thus tear
    # the whole RP stack down via a 'session.close()' call in the 'finally'
    # clause...
    try:

# !!!   you may need to specify a login name below, to be used in the session.
        # Add an ssh identity to the session.
        c = rp.Context('ssh')
      # c.user_id = 'osdcXX'
        session.add_context(c)

        # Add a Pilot Manager. Pilot managers manage one or more ComputePilots.
        print "Initializing Pilot Manager ..."
        pmgr = rp.PilotManager(session=session)

        # Register our callback with the PilotManager. This callback will get
        # called every time any of the pilots managed by the PilotManager
        # change their state.
        pmgr.register_callback(pilot_state_cb)

# !!!   you may want to specify a different target resource below
        # this describes the parameters and requirements for our pilot job
        pdesc = rp.ComputePilotDescription ()
        pdesc.resource = "xsede.stampede" # NOTE: This is a "label", not a hostname
        pdesc.runtime  = 5 # minutes
        pdesc.cores    = 8
        pdesc.cleanup  = True

# !!!   you may need to specify project and queue here
#       pdesc.project  = 'TG-MCB140109'
#       pdesc.queue    = 'default'

        # submit the pilot.
        print "Submitting Compute Pilot to Pilot Manager ..."
        pilot = pmgr.submit_pilots(pdesc)

        # Combine the ComputePilot, the ComputeUnits and a scheduler via
        # a UnitManager object.
        print "Initializing Unit Manager ..."
        umgr = rp.UnitManager (session=session,
                               scheduler=rp.SCHED_DIRECT_SUBMISSION)

        # Register our callback with the UnitManager. This callback will get
        # called every time any of the units managed by the UnitManager
        # change their state.
        umgr.register_callback(unit_state_cb)

        # Add the created ComputePilot to the UnitManager.
        print "Registering Compute Pilot with Unit Manager ..."
        umgr.add_pilots(pilot)

# !!!   # FIXME: change to a useful data file
        input     = '/data/random_1000000points.csv'
        output    = '$PWD'
        clusters  = 42
        threshold = 0.0010 # default
        cores     = 8      # Number of cores to use for once instance.

        cudesc = rp.ComputeUnitDescription()
        cudesc.cores       = cores
        cudesc.executable  = "kmeans_omp"
        cudesc.pre_exec    = ["module load intel-cluster-runtime"]
        cudesc.arguments   = ['-i', input,      # input file
                              '-z', output,     # output directory
                              '-n', clusters,   # number of clusters to find
                              '-t', threshold,  # convergence threshold
                              '-p', cores,      # number of cores, should be the same as cudesc.cores
                              '-o'              # output timing results
                              ]

        # Submit the previously created ComputeUnit descriptions to the
        # PilotManager. This will trigger the selected scheduler to start
        # assigning ComputeUnits to the ComputePilots.
        print "Submit Compute Unit to Unit Manager ..."
        cu_set = umgr.submit_units (cudesc)

        print "Waiting for CU to complete ..."
        umgr.wait_units()
        print "All CUs completed successfully!"


    except Exception as e:
        # Something unexpected happened in the pilot code above
        print "caught Exception: %s" % e
        raise

    except (KeyboardInterrupt, SystemExit) as e:
        # the callback called sys.exit(), and we can here catch the
        # corresponding KeyboardInterrupt exception for shutdown.  We also catch
        # SystemExit (which gets raised if the main threads exits for some other
        # reason).
        print "need to exit now: %s" % e

    finally:
        # always clean up the session, no matter if we caught an exception or
        # not.
        print "closing session"
        session.close ()

        # the above is equivalent to
        #
        #   session.close (cleanup=True, terminate=True)
        #
        # it will thus both clean out the session's database record, and kill
        # all remaining pilots (none in our example).


#-------------------------------------------------------------------------------

